
from django.conf import settings


PAGINATION_SETTINGS = getattr(settings, 'PAGINATION_SETTINGS', {})

PAGE_RANGE_DISPLAYED = PAGINATION_SETTINGS.get('PAGE_RANGE_DISPLAYED', 5)

MARGIN_PAGES_DISPLAYED = PAGINATION_SETTINGS.get('MARGIN_PAGES_DISPLAYED', 2)

SHOW_FIRST_PAGE_WHEN_INVALID = PAGINATION_SETTINGS.get(
        'SHOW_FIRST_PAGE_WHEN_INVALID', True)
